@@ -287,7 +287,6 @@ sub free_compiled_chunk {
    my $c = [$cx, $cy, $cz];
    my $id = world_pos2id ($c);
    my $l = delete $self->{compiled_chunks}->{$id};
-   delete $self->{dirty_chunks}->{$id};
    Games::Construder::Renderer::free_geom ($l) if $l;
    # WARNING FIXME XXX: this might not free up all chunks that were set/initialized by the server!
    Games::Construder::World::purge_chunk (@$c);
@@ -621,6 +620,7 @@ sub render_hud {
       next unless $_->{sticky};
       $_->display;
    }
+
    if (@{$self->{active_ui_stack}}) {
       $self->{active_ui_stack}->[-1]->[1]->display;
    }
@@ -712,6 +712,12 @@ sub setup_event_poller {
             #d# warn "freeed compiled chunk $kx, $ky, $kz\n";
          }
       }
+
+      for my $id (keys %{$self->{dirty_chunks}}) {
+         unless (exists $self->{compiled_chunks}->{$_}) {
+            delete $self->{dirty_chunks}->{$_};
+         }
+      }
    };
 
    my $anim_ltime;
@@ -1197,7 +1203,7 @@ sub show_credits {
       map {
          ref $_
             ? (ui_subdesc ("* $_->[0]", font => "small"),
-               ui_small_text ($_->[1], align => "center"))
+               ui_small_text ($_->[1], align => "center", wrap => 100))
             : ui_subdesc ($_, font => "small")
       } @{$si->{credits}}
    ));
